/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ISTRATEGY_HH
#define ISTRATEGY_HH

class IStrategy
{
  public:
    /// Destruktor.
    /**
     * Muss definiert werden, damit es bei Ableitungen
     * kein Problem gibt.
     */
    virtual ~IStrategy() { };

    /// Akzeptiert oder lehnt ein Angebot ab.
    /**
     * Diese Methode berechnet, ob der Wert angenommen oder abgelehnt
     * wird. Der angebotene Wert muss zwischen 0 und 1000 liegen.
     * Die Methode muss von den Ableitungen implementiert werden!
     * @param value Der angebotene Wert.
     * @return true, wenn das Angebot akzeptiert wurde, ansonsten false
     */
    virtual bool acceptOffer( const int value ) = 0;

    /// Erstellt eine Kopie des Elements.
    /**
     * Wenn man abgeleitete Klassen von einer Basis-Klasse hat
     * und nur mit einem Pointer auf die Basis-Klasse arbeitet,
     * lassen sich Objekte nicht so einfach per Zuweisung kopieren
     * da man ja die exakte Klassen benötigt. In dem Fall arbeitet
     * man mit einer virtuellen copy-Methode, welche in jeder
     * abgeleiteten Klasse implementiert werden muss.
     * @return Pointer auf die Kopie des Objekt.
     */
    virtual IStrategy* copy() const = 0;
};

#endif // ISTRATEGY_HH
